//
// file: primitive3D_searchLightAltA.fx
// version: v1.5
// author: Ren712
//

//--------------------------------------------------------------------------------------
// Settings
//--------------------------------------------------------------------------------------
float3 sElementPosition = float3(0, 0, 0);
float3 sElementRotation = float3(0, 0, 0);
float4 sElementColor = float4(0, 0, 0, 0);
float sLightPhi = 0.5; // Phi is the outer cone angle
float sLightTheta = 0.3; // Theta is the inner cone angle
float sLightFalloff = 1; // light intensity attenuation between the phi and theta areas
float sLightAttenuation = 10; // light attenuation (max radius)
float sLightAttenuationPower = 2; // light attenuation power

float sCoronaRescale = 1;
float sCoronaDepthBias = 0.2;
float sCoronaScaleSpread = 0.85;

float2 gDistFade = float2(250, 150);

//--------------------------------------------------------------------------------------
// Variables set by MTA
//--------------------------------------------------------------------------------------
float4x4 gProjection : PROJECTION;
float4x4 gView : VIEW;
float4x4 gViewInverse : VIEWINVERSE;
float3 gCameraPosition : CAMERAPOSITION;

static const float PI = 3.14159265f;
int CUSTOMFLAGS < string skipUnusedParameters = "yes"; >;

//--------------------------------------------------------------------------------------
// Structures
//--------------------------------------------------------------------------------------
struct VSInput
{
    float3 Position : POSITION0;
    float2 TexCoord : TEXCOORD0;
    float4 Diffuse : COLOR0;
};

struct PSInput
{
    float4 Position : POSITION0;
    float2 TexCoord : TEXCOORD0;
    float3 WorldPos : TEXCOORD1;
    float3 LightDir : TEXCOORD2;
    float DistFade : TEXCOORD3;
    float4 Diffuse : COLOR0;
};

//--------------------------------------------------------------------------------------
// Create world matrix with world position and ZXY rotation
//--------------------------------------------------------------------------------------
float4x4 createWorldMatrix(float3 pos, float3 rot)
{
    float4x4 eleMatrix = {
        float4(cos(rot.z) * cos(rot.y) - sin(rot.z) * sin(rot.x) * sin(rot.y), 
                cos(rot.y) * sin(rot.z) + cos(rot.z) * sin(rot.x) * sin(rot.y), -cos(rot.x) * sin(rot.y), 0),
        float4(-cos(rot.x) * sin(rot.z), cos(rot.z) * cos(rot.x), sin(rot.x), 0),
        float4(cos(rot.z) * sin(rot.y) + cos(rot.y) * sin(rot.z) * sin(rot.x), sin(rot.z) * sin(rot.y) - 
                cos(rot.z) * cos(rot.y) * sin(rot.x), cos(rot.x) * cos(rot.y), 0),
        float4(pos.x,pos.y,pos.z, 1),
    };
    return eleMatrix;
}

//--------------------------------------------------------------------------------------
// Return a rotation matrix (rotate by Y)
//--------------------------------------------------------------------------------------
float4x4 makeYRotation( float angleInRadians) 
{
  float c = cos(angleInRadians);
  float s = sin(angleInRadians);

  return float4x4(
    c, 0, -s, 0,
    0, 1, 0, 0,
    s, 0, c, 0,
    0, 0, 0, 1
  );
};

//--------------------------------------------------------------------------------------
// Vertex Shader 
//--------------------------------------------------------------------------------------
PSInput VertexShaderFunction(VSInput VS)
{
    PSInput PS = (PSInput)0;
	
    // set proper position and scale of the quad
    VS.Position.xyz = float3(- 0.5 + VS.TexCoord.xy, 0);
	
	VS.TexCoord.y = 1 - VS.TexCoord.y;
	
    // scale the quad
    VS.Position.xy *= tan(sLightPhi) * sLightAttenuation * sCoronaRescale;
    VS.TexCoord.x = 1 - VS.TexCoord.x;

    // create WorldMatrix for the quad
    float4x4 sWorld = createWorldMatrix(sElementPosition, sElementRotation);
	
    // move the billboard a bit forward
    sWorld[3].xyz += sWorld[1].xyz * sCoronaDepthBias;
	
    // get distance from front plane
    float planeZDist = dot(sWorld[1].xyz, gCameraPosition - sElementPosition);
	
    // angle between front plane and view
    float angleZ = acos(dot(normalize(sElementPosition - gCameraPosition), sWorld[1].xyz));
	
    // deal with front/back case	
    angleZ = planeZDist > 0 ? (PI - angleZ) : PI / 2;
	
    float impactZ = pow( smoothstep(sLightPhi, sLightTheta, angleZ ), sLightFalloff);
    if (angleZ > sLightPhi) impactZ = 0;
	
    // scale the quad based on impactZ
    VS.Position.xy *= (1 - sCoronaScaleSpread) + impactZ * sCoronaScaleSpread;

    // calculate screen position of the vertex
    float4x4 sWorldView = mul(sWorld, gView);
    float3 positionVS = VS.Position.xyz + sWorldView[3].xyz;
    PS.Position = mul(float4(positionVS, 1), gProjection);
	
    // get quad size
    float2 elementSize = float2(2 * tan(sLightPhi) * sLightAttenuation, sLightAttenuation);

    // get clip values
    float nearClip = - gProjection[3][2] / gProjection[2][2];
    float farClip = (gProjection[3][2] / (1 - gProjection[2][2]));	
	
    // fade corona
    float DistFromCam = distance(gCameraPosition, sElementPosition.xyz);
    float elementMedSize = (elementSize.x + elementSize.y) / 2;
    float2 DistFade = float2(min(gDistFade.x, farClip - elementMedSize / 2), min(gDistFade.y, farClip - elementMedSize /2 - (gDistFade.x - gDistFade.y)));
    PS.DistFade = saturate((DistFromCam - DistFade.x)/(DistFade.y - DistFade.x));
	
    // pass texCoords and vertex color to PS
    PS.TexCoord = VS.TexCoord;
    PS.Diffuse = VS.Diffuse * sElementColor;

    return PS;
}

//--------------------------------------------------------------------------------------
// Pixel shaders 
//--------------------------------------------------------------------------------------
float4 PixelShaderFunction(PSInput PS) : COLOR0
{
    // compute the direction to the light
    float zDistance = tan(sLightPhi) * sLightAttenuation;
    float3 vLight = normalize(float3((PS.TexCoord.xy - 0.5) * zDistance, sLightAttenuation / 3));

    // determine the angle between the current sample
    // and the light's direction:
    float angle = acos(dot(-vLight, float3(0,0,-1)));
	
    // compute the spot attenuation factor
    float fSpotAtten = 0.0f;
    if ( angle > sLightPhi ) fSpotAtten = 0.0f;
    else if ( angle < sLightTheta) fSpotAtten = 1.0f;
    else fSpotAtten = pow( smoothstep(sLightPhi, sLightTheta, angle ), sLightFalloff);
	
    // get vertex color
    float4 finalColor = PS.Diffuse;

    // multiply by attenuation and distFade
    finalColor.a *= saturate(fSpotAtten);
    finalColor.a *= saturate(PS.DistFade);
    finalColor.a *= 0.8;

    return saturate(finalColor);
}

//--------------------------------------------------------------------------------------
// Techniques
//--------------------------------------------------------------------------------------
technique dxDrawPrimitive3Dfx_searchLightAltA
{
  pass P0
  {
    ZEnable = true;
    ZFunc = LessEqual;
    ZWriteEnable = false;
    CullMode = 1;
    ShadeMode = Gouraud;
    AlphaBlendEnable = true;
    SrcBlend = SrcAlpha;
    DestBlend = One;
    AlphaTestEnable = true;
    AlphaRef = 1;
    AlphaFunc = GreaterEqual;
    Lighting = false;
    FogEnable = false;
    VertexShader = compile vs_2_0 VertexShaderFunction();
    PixelShader  = compile ps_2_0 PixelShaderFunction();
  }
} 

// Fallback
technique fallback
{
  pass P0
  {
    // Just draw normally
  }
}
